@@ -24,11 +24,6 @@ extern "C" {
 #include <fcntl.h>
 }
 
-// The maximum number of frames to allow in either the filter or encode queue.
-// Note that this needs to be large enough for the encoder to be able to do
-// intra-frame analysis.
-static constexpr int MaxQueueSize = 50;
-
 Q_DECLARE_METATYPE(std::optional<int>);
 Q_DECLARE_METATYPE(std::optional<std::chrono::nanoseconds>);
 
@@ -51,6 +46,17 @@ void PipeWireProduce::initialize()
 {
     m_stream.reset(new PipeWireSourceStream(nullptr));
     m_stream->setMaxFramerate(m_frameRate);
+
+    // The check in supportsHardwareEncoding() is insufficient to fully
+    // determine if we actually support hardware encoding the current stream,
+    // but to determine that we need the stream size, which we don't get until
+    // after we've created the stream, but creating the stream sets important
+    // parameters that require the correct usage hint to be set. So use the
+    // insufficient check to set the hint, assuming that we still get a working
+    // stream when we use the wrong hint with software encoding.
+    m_stream->setUsageHint(Encoder::supportsHardwareEncoding() ? PipeWireSourceStream::UsageHint::EncodeHardware
+                                                               : PipeWireSourceStream::UsageHint::EncodeSoftware);
+
     bool created = m_stream->createStream(m_nodeId, m_fd);
     if (!created || !m_stream->error().isEmpty()) {
         qCWarning(PIPEWIRERECORD_LOGGING) << "failed to set up stream for" << m_nodeId << m_stream->error();
@@ -63,14 +69,29 @@ void PipeWireProduce::initialize()
 
 Fraction PipeWireProduce::maxFramerate() const
 {
-    return m_stream->framerate();
+    return m_maxFramerate;
 }
 
 void PipeWireProduce::setMaxFramerate(const Fraction &framerate)
 {
+    m_maxFramerate = framerate;
     m_stream->setMaxFramerate(framerate);
 }
 
+int PipeWireProduce::maxPendingFrames() const
+{
+    return m_maxPendingFrames;
+}
+
+void PipeWireProduce::setMaxPendingFrames(int newMaxBufferSize)
+{
+    if (newMaxBufferSize < 3) {
+        qCWarning(PIPEWIRERECORD_LOGGING) << "Maxmimum pending frame count of " << newMaxBufferSize << " requested. Value must be 3 or higher.";
+        newMaxBufferSize = 3;
+    }
+    m_maxPendingFrames = newMaxBufferSize;
+}
+
 void PipeWireProduce::setupStream()
 {
     qCDebug(PIPEWIRERECORD_LOGGING) << "Setting up stream";
@@ -100,7 +121,7 @@ void PipeWireProduce::setupStream()
                 break;
             }
 
-            auto [filtered, queued] = m_encoder->encodeFrame(MaxQueueSize - m_pendingEncodeFrames);
+            auto [filtered, queued] = m_encoder->encodeFrame(m_maxPendingFrames - m_pendingEncodeFrames);
             m_pendingFilterFrames -= filtered;
             m_pendingEncodeFrames += queued;
 
@@ -144,6 +165,15 @@ void PipeWireProduce::setQuality(const std::optional<quint8> &quality)
     }
 }
 
+void PipeWireProduce::setEncodingPreference(const PipeWireBaseEncodedStream::EncodingPreference &encodingPreference)
+{
+    m_encodingPreference = encodingPreference;
+
+    if (m_encoder) {
+        m_encoder->setEncodingPreference(encodingPreference);
+    }
+}
+
 void PipeWireProduce::processFrame(const PipeWireFrame &frame)
 {
     auto f = frame;
@@ -162,7 +192,12 @@ void PipeWireProduce::processFrame(const PipeWireFrame &frame)
         return;
     }
 
-    if (m_pendingFilterFrames + 1 > MaxQueueSize) {
+    auto frameTime = 1000.0 / (m_maxFramerate.numerator / m_maxFramerate.denominator);
+    if ((pts - m_previousPts) < frameTime) {
+        return;
+    }
+
+    if (m_pendingFilterFrames + 1 > m_maxPendingFrames) {
         qCWarning(PIPEWIRERECORD_LOGGING) << "Filter queue is full, dropping frame" << pts;
         return;
     }
@@ -250,10 +285,10 @@ std::unique_ptr<Encoder> PipeWireProduce::makeEncoder()
     case PipeWireBaseEncodedStream::H264Baseline:
     case PipeWireBaseEncodedStream::H264Main: {
         auto profile = m_encoderType == PipeWireBaseEncodedStream::H264Baseline ? Encoder::H264Profile::Baseline : Encoder::H264Profile::Main;
-
         if (!forceSoftware) {
             auto hardwareEncoder = std::make_unique<H264VAAPIEncoder>(profile, this);
             hardwareEncoder->setQuality(m_quality);
+            hardwareEncoder->setEncodingPreference(m_encodingPreference);
             if (hardwareEncoder->initialize(size)) {
                 return hardwareEncoder;
             }
@@ -262,6 +297,7 @@ std::unique_ptr<Encoder> PipeWireProduce::makeEncoder()
         if (!forceHardware) {
             auto softwareEncoder = std::make_unique<LibX264Encoder>(profile, this);
             softwareEncoder->setQuality(m_quality);
+            softwareEncoder->setEncodingPreference(m_encodingPreference);
             if (softwareEncoder->initialize(size)) {
                 return softwareEncoder;
             }
