@@ -142,24 +142,22 @@ bool PipeWireRecordProduce::setupFormat()
 void PipeWireRecordProduce::processFrame(const PipeWireFrame &frame)
 {
     PipeWireProduce::processFrame(frame);
-    if (frame.cursor && !frame.dmabuf && !frame.image && !m_frameWithoutMetadataCursor.isNull()) {
-        PipeWireFrame frame;
-        frame.image = m_frameWithoutMetadataCursor;
-        m_encoder->filterFrame(frame);
+    if (frame.cursor && !frame.dmabuf && !frame.dataFrame && m_frameWithoutMetadataCursor.dataFrame) {
+        m_encoder->filterFrame(m_frameWithoutMetadataCursor);
     }
 }
 
 void PipeWireRecordProduce::aboutToEncode(PipeWireFrame &frame)
 {
-    if (!frame.image) {
+    if (!frame.dataFrame) {
         return;
     }
 
     if (m_cursor.position && !m_cursor.texture.isNull()) {
-        auto &image = *frame.image;
+        auto image = frame.dataFrame->toImage();
         // Do not copy the image if it's already ours
-        if (m_frameWithoutMetadataCursor.cacheKey() != image.cacheKey()) {
-            m_frameWithoutMetadataCursor = image.copy();
+        if (m_frameWithoutMetadataCursor.dataFrame->cleanup != frame.dataFrame->cleanup) {
+            m_frameWithoutMetadataCursor.dataFrame = frame.dataFrame->copy();
         }
         QPainter p(&image);
         p.drawImage(*m_cursor.position, m_cursor.texture);
