@@ -8,6 +8,7 @@
 
 #pragma once
 
+#include <QDebug>
 #include <QHash>
 #include <QImage>
 #include <QObject>
@@ -25,6 +26,7 @@
 
 #undef Status
 
+class PipeWireFrameCleanupFunction;
 class PipeWireCore;
 struct gbm_device;
 
@@ -58,16 +60,32 @@ struct PipeWireCursor {
         return position == other.position && hotspot == other.hotspot && texture == other.texture;
     }
 };
-Q_DECLARE_METATYPE(PipeWireCursor);
+
+class KPIPEWIRE_EXPORT PipeWireFrameData
+{
+    Q_DISABLE_COPY(PipeWireFrameData)
+public:
+    PipeWireFrameData(spa_video_format format, void *data, QSize size, qint32 stride, PipeWireFrameCleanupFunction *cleanup);
+    ~PipeWireFrameData();
+
+    QImage toImage() const;
+    std::shared_ptr<PipeWireFrameData> copy() const;
+
+    const spa_video_format format;
+    void *const data = nullptr;
+    const QSize size;
+    const qint32 stride = 0;
+    PipeWireFrameCleanupFunction *const cleanup = nullptr;
+};
 
 struct KPIPEWIRE_EXPORT PipeWireFrame {
     spa_video_format format;
-    std::optional<int> sequential;
+    std::optional<quint64> sequential;
     std::optional<std::chrono::nanoseconds> presentationTimestamp;
     std::optional<DmaBufAttributes> dmabuf;
-    std::optional<QImage> image;
     std::optional<QRegion> damage;
     std::optional<PipeWireCursor> cursor;
+    std::shared_ptr<PipeWireFrameData> dataFrame;
 };
 
 struct Fraction {
@@ -87,14 +105,23 @@ struct Fraction {
     quint32 denominator = 0;
 };
 
-KPIPEWIRE_EXPORT QImage::Format SpaToQImageFormat(quint32 /*spa_video_format*/ format);
-
 struct PipeWireSourceStreamPrivate;
 
 class KPIPEWIRE_EXPORT PipeWireSourceStream : public QObject
 {
     Q_OBJECT
 public:
+    /**
+     * A hint to indicate how this stream will be used.
+     *
+     * By default this will be set to Render.
+     */
+    enum class UsageHint {
+        Render, ///< Stream is intended mainly for reading pixel data and rendering directly to screen.
+        EncodeSoftware, ///< Stream is intended mainly for encoding video using software encoding.
+        EncodeHardware, ///< Stream is intended mainly for encoding video using hardware encoding.
+    };
+
     explicit PipeWireSourceStream(QObject *parent = nullptr);
     ~PipeWireSourceStream();
 
@@ -109,6 +136,9 @@ public:
     void setActive(bool active);
     void setDamageEnabled(bool withDamage);
 
+    UsageHint usageHint() const;
+    void setUsageHint(UsageHint hint);
+
     void handleFrame(struct pw_buffer *buffer);
     void process();
     void renegotiateModifierFailed(spa_video_format format, quint64 modifier);
@@ -138,5 +168,3 @@ private:
     void coreFailed(const QString &errorMessage);
     QScopedPointer<PipeWireSourceStreamPrivate> d;
 };
-
-Q_DECLARE_METATYPE(QList<DmaBufPlane>);
